/*
 *   coefmaker.c
 *
 *   coefmaker     Vers. 0.3.0
 *                 Copyright (C)  2001  Matteo Nastasi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *     CONTACT:    matteo.nastasi@milug.org
 *                 nastasi@alternativeoutput.it
 *
 
 Usage for the 32KHz to 44.1KHz convertion where the prefix for the 
 variable name is "resarr":
 
 ./coefmaker 320 441 resarr > converttab.h

*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define NPRELOOP        16
#define proxzero(a)     (fabs(a) < 0.00001 ? 0.0 : (a))


int main(int argc, char **argv)
{
  int e;
  double i, kin, kot, dec,inte, cur;
  char *name;
  FILE *fot = stdout;

  kin = atof(argv[1]);
  kot = atof(argv[2]);
  name = argv[3];
  fprintf(fot,"\n\n/* ==  coeff for preloop processing  ============================== */\n");
  fprintf(fot,"float %s%.0fto%.0fx[] = {\n",
	  name,kin,kot);
  for (i = 0 , e = 0 ; i <  NPRELOOP ; i++, e++) {
    cur = (kin*i) / kot;
    dec = modf(cur,&inte);
    inte = proxzero(inte);

    if (dec < 0.5 && inte > 0)
       inte--;
    fprintf(fot,"%15.10f",cur-inte);
    if (i < kot)
      fputc(',',fot);
    if (e % 4 == 3)
      fputc('\n', fot);
  }
  fprintf(fot,"};");
  fprintf(fot,"\n\n/* ==  indices for preloop processing  ============================== */\n");
  fprintf(fot,"int %s%.0fto%.0fc[] = {\n",
	  name,kin,kot);
  for (i = 0 , e = 0 ; i <=  NPRELOOP ; i++, e++) {
    cur = (kin*i) / kot;
    dec = modf(cur,&inte);
    inte = proxzero(inte);

    if (dec < 0.5 && inte > 0)
       inte--;
    fprintf(fot,"%3d",((int)inte)-2);
    if (i < kot)
      fputc(',',fot);
    if (e % 4 == 3 && e % 16 != 15)
      fputc(' ', fot);
    if (e % 16 == 15)
      fputc('\n', fot);
  }
  fprintf(fot,"};");
  fprintf(fot,"\n\n/* ==  coeff for loop processing  ================================= */\n");
  fprintf(fot,"float %s%.0fto%.0flx[] = {\n",
	  name,kin,kot);
  for (i = 0 , e = 0 ; i <=  kot ; i++, e++) {
    cur = (kin*i) / kot;
    dec = modf(cur,&inte);
    inte = proxzero(inte);

    if (dec < 0.5)
       inte--;
    fprintf(fot,"%15.10f",cur-inte);
    if (i < kot)
      fputc(',',fot);
    if (e % 4 == 3)
      fputc('\n', fot);
  }
  fprintf(fot,"};");
  fprintf(fot,"\n\n/* ==  indices for loop processing  ================================= */\n");
  fprintf(fot,"int %s%.0fto%.0flc[] = {\n",
	  name,kin,kot);
  for (i = 0 , e = 0 ; i <=  kot ; i++, e++) {
    cur = (kin*i) / kot;
    dec = modf(cur,&inte);
    inte = proxzero(inte);

    if (dec < 0.5)
       inte--;
    fprintf(fot,"%3d",((int)inte)-2);
    if (i < kot)
      fputc(',',fot);
    if (e % 4 == 3 && e % 16 != 15)
      fputc(' ', fot);
    if (e % 16 == 15)
      fputc('\n', fot);
  }
  fprintf(fot,"};\n");
  fprintf(fot,"\n/* end */");


  exit(0);
  return(0);
}
